/*
 * Copyright 2016 Alexander Baumgartner
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package at.jku.risc.stout.urmc.algo;

import at.jku.risc.stout.urmc.data.TermNode;
import at.jku.risc.stout.urmc.data.atom.Variable;

public class MatchEquation {
	public static String PRINT_EQ_SEPARATOR = " << ";
	private TermNode left, right;
	private MatchEquation next;

	public MatchEquation(TermNode left, TermNode right) {
		this.left = left;
		this.right = right;
	}

	public TermNode getLeft() {
		return left;
	}

	public void setLeft(TermNode left) {
		this.left = left;
	}

	public TermNode getRight() {
		return right;
	}

	public void setRight(TermNode right) {
		this.right = right;
	}

	public MatchEquation getNext() {
		return next;
	}

	public void setNext(MatchEquation next) {
		this.next = next;
	}

	public void apply(Substitution theta) {
		left = left.apply(theta.getMapping());
	}

	public void apply(Variable fromVar, TermNode toTerm) {
		left = left.substitute(fromVar, toTerm);
	}

	public MatchEquation copy() {
		MatchEquation cpy = new MatchEquation(left.copy(), right.copy());
		cpy.next = next;
		return cpy;
	}

	@Override
	public int hashCode() {
		return (left.hashCode() + 1) ^ right.hashCode();
	}

	@Override
	public boolean equals(Object obj) {
		if (obj instanceof MatchEquation)
			return left.equals(((MatchEquation) obj).left)
					&& right.equals(((MatchEquation) obj).right);
		return false;
	}

	@Override
	public String toString() {
		return left.toString() + PRINT_EQ_SEPARATOR + right.toString();
	}

	public void swap() {
		TermNode left = this.left;
		this.left = this.right;
		this.right = left;
	}
}
